/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.dispatcher;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.eclipse.stp.sca.Composite;
import org.eclipse.stp.sca.ContributionType;
import org.eclipse.stp.sca.DeployableType;
import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.ow2.frascati.factory.core.instance.InstantiationException;
import org.ow2.frascati.factory.core.instance.NotSupportedException;
import org.ow2.frascati.factory.core.instance.assembly.ScaAssembly;
import org.ow2.frascati.factory.core.instance.runtime.RuntimeFactory;
import org.ow2.frascati.factory.core.parser.Sca2Ecore;

/**
 * Manage Assembly Factory instantiation process
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@Component
public class ManagerImpl implements Manager {

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(ManagerImpl.class.getCanonicalName());

  /**
   * The required model component
   */

  @Requires(name = "model")
  public Sca2Ecore model;

  /**
   * The required assembly builder component
   */

  @Requires(name = "assembly")
  public ScaAssembly assembly;

  /**
   * The required runtime
   */

  @Requires(name = "runtime")
  public RuntimeFactory runtime;

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.dispatcher.Manager#loadComposite(java.lang
   * .String)
   */

  public Object loadComposite(String composite) throws NotSupportedException,
      InstantiationException {

    // SCA Composite instance given by EMF
    Composite compositeModel = null;

    // Component Object return by factory
    Object componentObject = null;

    // Load composite as URI using internal Loader
    URI compositeFileURI = runtime.getCompositeResource(composite);

    // Call the EMF parser component
    log.fine("Reading composite " + composite);

    try {
      // Use SCA parser to create composite model instance
      compositeModel = model.parseComposite(compositeFileURI);
    } catch (IOException e) {
      log.severe("Error when loading the composite file " + composite
          + " with the SCA XML Processor");
      e.printStackTrace();
      System.exit(1);
    }

    // Return the resulting assembly object (fractal composite)

    componentObject = assembly.create(compositeModel);

    return componentObject;
  }

  /**
   * Load components according to a sca contribution
   * 
   * @param contibution
   *          - contribution descriptor
   * @return loaded components from contribution
   * @throws InstantiationException
   * @throws NotSupportedException
   */
  private org.objectweb.fractal.api.Component[] loadContribution(URI contibution)
      throws NotSupportedException, InstantiationException {

    ArrayList<org.objectweb.fractal.api.Component> components = new ArrayList<org.objectweb.fractal.api.Component>();

    // SCA contribution instance given by EMF
    ContributionType contributionModel = null;

    // Load composite as URI using internal Loader
    // URI contributionFileURI = runtime.getCompositeResource(contibution);

    // Call the EMF parser component
    log.fine("Reading contribution " + contibution);

    try {
      // Use SCA parser to create contribution model instance
      contributionModel = model.parseContribution(contibution);
    } catch (IOException e) {
      log.severe("Error when loading the contribution file " + contibution
          + " with the SCA XML Processor");
      log.severe(e.getMessage());
    }
    // Iterate over 'Deployable' defined in contribution descriptor
    for (DeployableType deployable : contributionModel.getDeployable()) {

      // Retrieve 'deployable' composite name
      String compositeName = deployable.getComposite().getLocalPart();

      // Retrieve file URI if prefix is present
      if (!deployable.getComposite().getPrefix().equals("")) {
        String prefixUri = deployable.getComposite().getNamespaceURI();

        // Check if namespace uri ends with "/"
        if (!prefixUri.endsWith("/"))
          prefixUri = prefixUri + "/";

        // Concatenate URI string
        compositeName = prefixUri + compositeName;
      }

      // Load, Start composite and add to list of loaded components for this
      // contribution
      org.objectweb.fractal.api.Component c = (org.objectweb.fractal.api.Component) loadComposite(compositeName);
      components.add(c);

    }
    // retrun loaded components
    return components
        .toArray(new org.objectweb.fractal.api.Component[components.size()]);
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.dispatcher.Manager#loadJars(java.net.URL[])
   */

  public ClassLoader loadJars(URL[] urls) {

    // Get list of URL for logging
    try {
      for (URL url : urls) {
        File test = new File(url.toURI());
        // Test if URL exist
        if (!test.exists())
          log.warning("File does not exist : " + url.toString());
        else
          log.info("Load library : " + url.toString());
      }
    } catch (URISyntaxException e) {
      e.printStackTrace();
    }

    runtime.loadLibraries(urls);

    return runtime.getClassLoader();
  }

  public void finalize() {
    this.model = null;
    this.assembly = null;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.dispatcher.Manager#getComposite(java.lang
   * .String, java.net.URL[])
   */

  public org.objectweb.fractal.api.Component getComposite(String composite,
      URL[] libs) throws NotSupportedException, InstantiationException {
    if (libs != null)
      loadJars(libs);

    return (org.objectweb.fractal.api.Component) loadComposite(composite);

  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.dispatcher.Manager#getContribution(java.lang
   * .String)
   */

  public org.objectweb.fractal.api.Component[] getContribution(
      String contribution) throws NotSupportedException, InstantiationException {

    try {
      // Array for found jar files
      ArrayList<URL> urls = new ArrayList<URL>();

      // sca-contribution.xml file
      URI scaContribution = null;

      // Load contribution zip file
      ZipFile zipFile = new ZipFile(contribution);

      // Get folder name for output
      final String folder = zipFile.getName().substring(
          zipFile.getName().lastIndexOf(File.separator),
          zipFile.getName().length() - 4);

      // Set directory for extracted files

      // TODO : use system temp direcory but should use output folder given by
      // runtime component. Will be possible once Assembly Factory modules will
      // be merged

      final String tempDir = System.getProperty("java.io.tmpdir")
          + File.separator + folder;

      Enumeration<? extends ZipEntry> entries = zipFile.entries();

      // Iterate over zip entries
      while (entries.hasMoreElements()) {

        ZipEntry entry = entries.nextElement();

        // create directories
        if (entry.isDirectory())

          new File(tempDir, entry.getName()).mkdirs();

        else {
          File f = new File(tempDir, File.separator + entry.getName());
          // register jar files
          if (entry.getName().endsWith("jar"))
            urls.add(f.toURL());
          // register contribution definition
          if (entry.getName().endsWith("sca-contribution.xml"))
            scaContribution = f.toURI();
          // extract entry from zip to tempDir
          copy(zipFile.getInputStream(entry), new BufferedOutputStream(
              new FileOutputStream(f)));

        }
      }

      if (scaContribution != null) {
        // Load jars
        loadJars(urls.toArray(new URL[urls.size()]));
        // Load contribution definition
        return loadContribution(scaContribution);
      }

    } catch (MalformedURLException e) {
      throw new org.ow2.frascati.factory.core.instance.InstantiationException(e);
    } catch (IOException e) {
      throw new org.ow2.frascati.factory.core.instance.InstantiationException(e);
    }

    return null;
  }

  /**
   * Stream copy - used to extract entries from contribution zip
   * 
   * @param in
   *          - input stream
   * @param out
   *          - output stream
   * @throws IOException
   */
  private void copy(InputStream in, OutputStream out) throws IOException {

    byte[] buffer = new byte[2048];
    int len;

    while ((len = in.read(buffer)) >= 0)
      out.write(buffer, 0, len);

    in.close();
    out.close();

  }

}
