/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Nicolas Dolet
 * 
 * Contributors: Christophe Demarey, Philippe Merle
 */

package org.ow2.frascati.factory.core.instance.binding;

import java.util.Map;
import org.eclipse.stp.sca.BaseReference;
import org.eclipse.stp.sca.BaseService;
import org.eclipse.stp.sca.Binding;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.bf.BindingFactory;
import org.objectweb.fractal.bf.BindingFactoryException;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.util.Fractal;
import org.ow2.frascati.factory.core.instance.runtime.RuntimeFactory;

/**
 * Abstract class for remote bindings implemented with the Fractal {@link BindingFactory}.
 * 
 * @author <a href="mailto:nicolas.dolet@inria.fr">Nicolas Dolet</a>
 * @version 1.1
 */
@org.objectweb.fractal.fraclet.annotations.Component
public abstract class AbstractRemoteBinding<BindingType extends Binding>
           implements ScaBindingProtocol {
  
  /**
   * The required binding factory.
   */
  @Requires(name = "binding-factory")
  private BindingFactory bf;
  
  /**
   * The required runtime.
   */
  @Requires(name = "runtime")
  protected RuntimeFactory runtime;

  /**
   * Logger.
   */
  protected java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(getClass().getCanonicalName());
  
  //---------------------------------------------------------------------------
  // Abstract methods defined in concrete subclasses.
  // --------------------------------------------------------------------------

  /**
   * Get the Fractal {@link BindingFactory} plugin to use.
   *
   * @return the Fractal {@link BindingFactory} plugin to use.
   */
  public abstract String getBindingFactoryPluginId();

  /**
   * Initialize hints for a concrete {@link BindingType}.
   * @param b the binding
   * @param hints a {@link Map} representing hints to initialize with
   *   <ul>
   *     <li> key: the Object id represented with a {@link String}
   *     <li> value: the {@link Object}
   *   </ul>
   */
  public abstract void initializeBindingHints(BindingType binding,
                                              Map<String, Object> hints);
  
  //---------------------------------------------------------------------------
  // Implementation of the ScaBindingProtocol interface.
  // --------------------------------------------------------------------------

  public void bind(BaseReference reference, Binding binding, Component component) {
    
    String referenceName = ScaBindingUtil.getReferenceName(reference);
    Map<String, Object> hints = createBindingFactoryHints(binding);
    try {
      log.fine("Calling binding factory\n bind : "
          + Fractal.getNameController(component).getFcName() + " -> "
          + referenceName);
      bf.bind(component, referenceName, hints);
    } catch (BindingFactoryException e) {
      log.severe("Error while binding reference: " + referenceName);
      e.printStackTrace();
    } catch (NoSuchInterfaceException e) {
      log.severe("Error while getting component name controller");
      e.printStackTrace();
    }
  }

  public void export(BaseService service, Binding binding, Component component) {
    
    String serviceName = ScaBindingUtil.getServiceName(service);
    Map<String, Object> hints = createBindingFactoryHints(binding);
    try {
      log.fine("Calling binding factory\n export : "
          + Fractal.getNameController(component).getFcName() + " -> "
          + serviceName);
      bf.export(component, serviceName, hints);
    } catch (BindingFactoryException e) {
      log.severe("Error while binding service: " + serviceName);
      e.printStackTrace();
    } catch (NoSuchInterfaceException e) {
      log.severe("Error while getting component name controller");
      e.printStackTrace();
    }
  }

  //---------------------------------------------------------------------------
  // Internal methods.
  // --------------------------------------------------------------------------

  /**
   * Get EMF ID.
   */
  protected static String getID(org.eclipse.emf.ecore.EClass eclass) {
    return org.ow2.frascati.factory.core.parser.util.ScaModelHelper.getID(eclass);
  }

  /**
   * Create hints for the Fractal {@link BindingFactory}.
   */
  @SuppressWarnings("unchecked")
  private Map<String, Object> createBindingFactoryHints(Binding binding) {
    Map<String, Object> hints = new java.util.HashMap<String, Object>();
    // set used binding protocol.
    hints.put(ScaBindingProtocol.PLUGIN_ID, getBindingFactoryPluginId());
    // set the class loader to use.
    hints.put(ScaBindingProtocol.CLASSLOADER, runtime.getClassLoader());
    // initialize hints specific to the binding type.
    initializeBindingHints((BindingType)binding, hints);
    return hints;
  }
}
