/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 * 
 * Contributor(s): Nicolas Dolet
 */

package org.ow2.frascati.factory.core.instance.binding;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.stp.sca.BaseReference;
import org.eclipse.stp.sca.BaseService;
import org.eclipse.stp.sca.Binding;
import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.types.Cardinality;
import org.objectweb.fractal.fraclet.types.Policy;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.frascati.factory.core.instance.NotSupportedException;
import org.ow2.frascati.factory.core.parser.util.ScaModelHelper;

/**
 * Binding protocol switcher component - Select the corresponding binding
 * protocol
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@Component
public class ScaBindingSwitch implements ScaBinding {

  @Requires(name = "bindings", cardinality = Cardinality.COLLECTION)
  private Map<String, ScaBindingProtocol> bindingPlugins = new HashMap<String, ScaBindingProtocol>();

  // Map providing direct access to binding plugins
  private Map<String, ScaBindingProtocol> bindingIDMap = new HashMap<String, ScaBindingProtocol>();

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(ScaBindingSwitch.class.getCanonicalName());

  /**
   * Binding Switch initializer, Build map for accessing binding plugins
   * according to the Binding ID
   */

  @Lifecycle(step=Step.START, policy=Policy.ASYNCHRONOUS)
  public void initialize() {
    // construct map for accessing binding plugins using EMF Classifier ID
    Set<String> keys = bindingPlugins.keySet();
    for (String key : keys) {
      bindingIDMap.put(bindingPlugins.get(key).getBindingID(), bindingPlugins
          .get(key));
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.ow2.frascati.factory.core.instance.binding.ScaBinding#bind(org.eclipse.stp.sca.BaseReference,
   *      org.objectweb.fractal.api.Component)
   */

  public void bind(BaseReference reference,
      org.objectweb.fractal.api.Component component) throws NotSupportedException {

    Iterator<Binding> bindings = reference.getBinding().iterator();

    while (bindings.hasNext()) {
      Binding binding = bindings.next();
      // get binding plugin
      String id = ScaModelHelper.getID(binding);
      ScaBindingProtocol protocol = this.bindingIDMap.get(id);
      if (protocol != null)
        protocol.bind(reference, binding, component);
      else
        throw new NotSupportedException("Binding type not supported : " + binding.eClass().getName());
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.ow2.frascati.factory.core.instance.binding.ScaBinding#export(org.eclipse.stp.sca.BaseService,
   *      org.objectweb.fractal.api.Component)
   */

  public void export(BaseService service,
      org.objectweb.fractal.api.Component component) throws NotSupportedException {

    Iterator<Binding> bindings = service.getBinding().iterator();

    while (bindings.hasNext()) {
      Binding binding = bindings.next();
      // get binding plugin
      String id = ScaModelHelper.getID(binding);
      ScaBindingProtocol protocol = this.bindingIDMap.get(id);
      if (protocol != null)
        protocol.export(service, binding, component);
      else
        throw new NotSupportedException("Binding type not supported : " + binding.eClass().getName());
    }
  }

}
