/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Philippe Merle
 * Contributor(s) : Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.implementation.spring;

import org.eclipse.stp.sca.Implementation;
import org.eclipse.stp.sca.ScaPackage;
import org.eclipse.stp.sca.SpringImplementation;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.util.Fractal;
import org.ow2.frascati.factory.core.instance.implementation.ImplementationException;
import org.ow2.frascati.factory.core.instance.implementation.ScaImplementation;
import org.ow2.frascati.factory.core.instance.runtime.RuntimeFactory;
import org.ow2.frascati.factory.core.parser.util.ScaModelHelper;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.ow2.frascati.factory.core.instance.InstantiationException;

/**
 * Create FraSCAti component instances implemented with the Spring Framework.
 * 
 * @author <a href="mailto:Philippe.Merle@inria.fr">Philippe Merle</a>
 */

@org.objectweb.fractal.fraclet.annotations.Component
public class ScaSpringImpl implements ScaImplementation {
  // ======================================================================
  //
  // Internal state.
  //
  // ======================================================================
  
  /**
   * The implementation type calculated with Object Eclass
   */

  public static String IMPLEMENTATION_TYPE = ScaModelHelper.getID(ScaPackage.Literals.SPRING_IMPLEMENTATION);
  
  /**
   * Logger for this class.
   */
  protected java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(this.getClass().getCanonicalName());

  /**
   * The Fractal generic factory used to create Fractal components wrapping
   * Spring beans.
   */
  @Requires(name = "generic-factory")
  protected RuntimeFactory genericFactory;

  // ======================================================================
  //
  // Constructor.
  //
  // ======================================================================

  // ======================================================================
  //
  // Implementation of
  // org.ow2.frascati.factory.core.instance.implementation.ScaImplementation
  //
  // ======================================================================

  /**
   * (non-Javadoc)
   * 
   * @see ScaImplementation#getImplementationID()
   */
  public String getImplementationID() {
    return IMPLEMENTATION_TYPE;
  }

  /**
   * (non-Javadoc)
   * @throws ImplementationException 
   * 
   * @see ScaImplementation#create(Implementation,ComponentType)
   */
  public Component create(final Implementation implementation,
      final ComponentType componentType) throws ImplementationException {
    // Implementation is a Spring one.
    SpringImplementation springImplementation = (SpringImplementation) implementation;
    // Get the Spring location.
    String springLocation = springImplementation.getLocation();

    log.finer("Create an SCA component with the Spring implementation "
        + springLocation + " and the Fractal component type "
        + componentType.toString());

    // TODO: Must handle location as described in the SCA specification.
    // location could be a Jar file or a directory.
    // Currently, location is considered to refer a Spring XML file in the class
    // path.

    Component component = null;
    try {
      // Create the Fractal component via the generic factory.
      component = genericFactory.newInstance(componentType, "scaComposite",
          null);
    } catch (InstantiationException ie) {
      // This should not happen!
      // TODO: Add an explicit exception to the 'create' method to report
      // problems!
      throw new ImplementationException("Internal Fractal error!", ie);
    }
    
    // Only if a component instance has been returned
    if (component != null) {
      // Create the Spring application context.
      // TODO: Must use another ApplicationContext to read XML files from JAR
      // files or directories.
      ClassPathXmlApplicationContext context = new ClassPathXmlApplicationContext(
          new String[] { springLocation }, new ParentApplicationContext(
              component));
      // It must use the current thread class loader to load Java classes.
      context.setClassLoader(Thread.currentThread().getContextClassLoader());

      // Traverse all the interface types of the component type.
      for (InterfaceType interfaceType : componentType.getFcInterfaceTypes()) {
        if (!interfaceType.isFcClientItf()) {
          // When this is a server interface type.
          String interfaceName = interfaceType.getFcItfName();
          // Get the Spring bean having the same name as the interface name.
          Object bean = context.getBean(interfaceName);
          try {
            // Bind the Fractal component to the Spring bean.
            Fractal.getBindingController(component).bindFc(interfaceName, bean);
          } catch (Exception exc) {
            // This should not happen!
            // TODO: Add an explicit exception to the 'create' method to report
            // problems!
            throw new Error("Internal Fractal error!", exc);
          }
        }
      }
    }
    // Return the Fractal component.
    return component;
  }
}
