/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Nicolas Dolet
 * 
 * Contributor(s):
 */

package org.ow2.frascati.factory.core.instance.intent;

import java.util.List;

import javax.xml.namespace.QName;

import org.eclipse.emf.ecore.EObject;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.util.Fractal;
import org.ow2.frascati.factory.core.instance.intent.api.Cache;
import org.ow2.frascati.tinfi.TinfiDomain;
import org.ow2.frascati.tinfi.control.intent.IntentHandler;
import org.ow2.frascati.tinfi.control.intent.SCABasicIntentController;

/**
 * Abstract {@link ScaIntent} class for FraSCAti intent components
 * 
 * @author <a href="mailto:nicolas.dolet@inria.fr">Nicolas Dolet</a>
 * @version $Revision$
 */
@org.objectweb.fractal.fraclet.annotations.Component
public abstract class AbstractIntentPlugin
           implements ScaIntent {
  
  @Requires
  protected Cache cache;

  //---------------------------------------------------------------------------
  // Abstract methods
  // --------------------------------------------------------------------------
  
  /**
   * Returns intents names for a given {@link EObject}
   * @param modelElement the element (component, binding or reference)
   * @return a {@link List} containing intents {@link QName} for <code>modelElement</code>
   */
  protected abstract List<QName> getIntentsFromModel(EObject modelElement);
  
  /**
   * Returns the name of an {@link EObject}
   * 
   * @param modelElement the element (component, binding or reference)
   * @return the name of <code>modelElement</code>
   */
  protected abstract String getName(EObject modelElement);

  /**
   * Register an intent handler to a controller for a given interface
   * 
   * @param ic the intent controller
   * @param h the intent handler
   * @param itf the interface name
   * @throws IllegalLifeCycleException if the intent cannot be added
   * @throws NoSuchInterfaceException if <code>itf</code> does not exist
   */
  protected abstract void addIntent(SCABasicIntentController ic,
      IntentHandler h, String itf) throws IllegalLifeCycleException,
      NoSuchInterfaceException;

  //---------------------------------------------------------------------------
  // Implementation of the ScaIntent interface
  // --------------------------------------------------------------------------
  
  public void setIntent(Object o, EObject modelElement) throws IntentException {

    // Retrieve fractal component
    Component component = (Component) o;

    // Get intent controller
    SCABasicIntentController ic;
    try {
      ic = (SCABasicIntentController) component
          .getFcInterface(SCABasicIntentController.NAME);
    } catch (NoSuchInterfaceException e) {
      throw new IntentException("Cannot access to intent controller",e);
    }

    List<QName> intents = getIntentsFromModel(modelElement);

    for (QName name : intents) {

      String intentName = name.getLocalPart();

      // Try to retrieve intent service from cache
      Component intent = cache.getFromCache(intentName);

      try {
        // Stop intent component
        Fractal.getLifeCycleController(intent).stopFc();

        // Get intent handler
        IntentHandler h = TinfiDomain.getService(intent,
            IntentHandler.class, "intent");

        // Add intent
        addIntent(ic, h, getName(modelElement));

        // Start intent component
        Fractal.getLifeCycleController(intent).startFc();

      } catch (Exception e) {
        throw new IntentException("Intent '" + intentName
            + "' cannot be added to component " + getName(modelElement), e);
      }
    }
  }
  
  public abstract int getEObjectID();
}
