/***
 * OW2 FraSCAti Explorer
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Christophe Demarey
 *
 * Contributor(s): Damien Fournier, Philippe Merle
 */

package org.ow2.frascati.explorer.action;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.filechooser.FileFilter;

import org.mortbay.log.Log;
import org.objectweb.fractal.api.Component;
import org.ow2.frascati.explorer.ExplorerGUI;
import org.ow2.frascati.factory.core.domain.DomainException;
import org.ow2.frascati.factory.runtime.domain.api.Domain;

import de.schlichtherle.io.swing.JFileChooser;

/**
 * This class provides the ability to load SCA composites
 * via the Load Menu item {@link MenuItem}.
 *
 * @author Christophe Demarey, Philippe Merle
 */

public class LoadMenuItem extends AbstractAlwaysEnabledMenuItem<Domain>
{
  // --------------------------------------------------------------------------
  // Internal state
  // --------------------------------------------------------------------------

  /** The {@link JFileChooser} instance. */
  protected static JFileChooser fileChooser = null;

  // --------------------------------------------------------------------------
  // Internal methods
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  // Constructor
  // --------------------------------------------------------------------------

  /**
   * The default constructor.
   */
  public LoadMenuItem () { }

  // --------------------------------------------------------------------------
  // AbstractAlwaysEnabledMenuItem
  // --------------------------------------------------------------------------

  /**
   * @see org.ow2.frascati.explorer.action.AbstractAlwaysEnabledMenuItem#execute()
   */
  protected void execute(final Domain domain) {

    // At the first call, creates the file chooser.
    if(fileChooser == null) {
      fileChooser = new JFileChooser();
      // Sets to the user's current directory.
      fileChooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
      // Filters SCA files.
      fileChooser.setFileFilter(            
        new FileFilter() {
          /** Whether the given file is accepted by this filter. */
          public boolean accept(File f) {
            String extension = f.getName().substring(f.getName().lastIndexOf('.') + 1);
            return extension.equalsIgnoreCase("composite") || f.isDirectory();
          }
          /** The description of this filter. */
          public String getDescription() {
            return "SCA composite files";
          }
        }
      );
    }

    // Opens the file chooser to select a file.
    int returnVal = fileChooser.showOpenDialog(null);
    if (returnVal == JFileChooser.APPROVE_OPTION) {
      File f = fileChooser.getSelectedFile();
      String name = f.toString(); 
      int index = name.indexOf(".jar");
      
      if ( index != -1 ) { // File in a jar
        // Add the jar to the classpath
        URL[] urls = new URL[1];
        String jarName = null;
        de.schlichtherle.io.File cfgFile = null;
        
        try {
          jarName = name.substring(0, index+4);
          urls[0] = new File(jarName).toURI().toURL();
          cfgFile = new de.schlichtherle.io.File(jarName+File.separator+ExplorerGUI.EXPLORER_STD_CFG_FILES_NAME);

          if ( cfgFile.exists() ) {
              Log.info("FraSCAti Explorer - Load " + cfgFile);
              ExplorerGUI.getSingleton().addExplorerConfigurationFile("jar:file:"+jarName+"!/"+ExplorerGUI.EXPLORER_STD_CFG_FILES_NAME);
          }
        } catch (MalformedURLException e1) {
          Log.warn(e1.getMessage());
        }
        
        // Load the composite
        loadComposite(domain, name.substring(index + 5), urls);
      } else {
        loadComposite(domain, f.getPath(), null);
      }
    }
  }
  
  // --------------------------------------------------------------------------
  // Public methods.
  // --------------------------------------------------------------------------

  /**
   * Load an SCA composite file.
   * 
   * @param domain - The domain that will host the composite.
   * @param compositeName - The name of the composite file.
   * @param urls - A list of libraries to add to the classpath.
   */
  public void loadComposite(Domain domain, String compositeName, URL[] urls) {
    Component c = null;

    try {
      if (urls == null)
        c = (Component) domain.getComposite(compositeName);
      else
        c = (Component) domain.getComposite(compositeName, urls);
    } catch (DomainException e) {
      e.printStackTrace();
      Log.warn("Cannot load "+compositeName+"!");
    }     
  }
}
