/***
 * OW2 FraSCAti explorer
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Christophe Demarey
 *
 * Contributor(s):
 * 
 */

package org.ow2.frascati.explorer.scaproperties;



/**
 * A class to represent SCA properties.
 * 
 * @author Christophe Demarey
 * @author Jakub Kornas
 */
public class PropertyDescriptor {
    private String propertyName = null;

    private Object propertyValue = null;

    private boolean isMutable = false;

    private Thread monitor = null;

    /**
     * @param propertyName the name of the property.
     * @param propertyValue the value of the property.
     * @param isMutable <code>true</code> if the property's value can be
     *         modified, <code>false</code> otherwise.
     */
    public PropertyDescriptor(String attributeName, Object attributeValue,
            boolean isMutable) {
        this.propertyName = attributeName;
        this.propertyValue = attributeValue;
        this.isMutable = isMutable;
    }

    /**
     * Returns the name of the attribute represented by this descriptor.
     * 
     * @return propertyName the name of the property.
     */
    public String getName() {
        return this.propertyName;
    }

    /**
     * Returns the value of the attribute represented by this descriptor.
     * 
     * @return propertyValue the value of the property.
     */
    public Object getValue() {
        return this.propertyValue;
    }

    /**
     * Tells whether a value of the attribute represented by this descriptor is
     * mutable (can be modified).
     * 
     * @return isMutable <code>true</code> if the property's value can be
     *         modified, <code>false</code> otherwise.
     */
    public boolean isMutable() {
        return this.isMutable;
    }

    /**
     * This method allows for the modification of the attribute's value.
     * 
     * @param newValue
     *            new value of the property represented by this descriptor.
     */
    public void setValue(Object newValue) {
        this.propertyValue = newValue;
    }

    /**
     * Tells whether the value of the property represented by this descriptor
     * is subject to monitoring, that is whether it is tested periodically for
     * modifications.
     * 
     * @return <code>true</code> if the property represented by this
     *         descriptor is subject to monitoring, <code>false</code>
     *         otherwise.
     */
    public boolean hasMonitor() {
        return monitor != null;
    }

    /**
     * Sets the monitor of the property represented by this descriptor to the
     * <code>Thread</code> passed as this method's parameter. The monitor is
     * responsible for periodically testing the value of the property
     * represented by this descriptor. The monitor has to be started externally
     * (its start method is not called in this method's body). If a monitor for
     * this descriptor already exists when this method is called, the "old"
     * monitor is stopped and destroyed.
     * 
     * @param monitor
     */
    public void setMonitor(Thread monitor) {
        if (hasMonitor()) {
            ((PropertyMonitoringThread) this.monitor).running = false;
            this.monitor = null;
        }
        this.monitor = monitor;
    }

    /**
     * Returns the <code>Thread</code> responsible for periodically testing
     * the value of the property corresponding to this descriptor.
     * 
     * @return monitor the thread responsible for periodically testing the value
     *         of the property corresponding to this descriptor.
     *         <code>null</code> if the monitor does not exist.
     */
    public Thread getMonitor() {
        return this.monitor;
    }
}
