/***
 * OW2 FraSCAti fscript
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Christophe Demarey
 */
package org.ow2.frascati.fscript.model;

import static org.objectweb.fractal.fscript.types.PrimitiveType.BOOLEAN;
import static org.objectweb.fractal.fscript.types.PrimitiveType.OBJECT;
import static org.objectweb.fractal.fscript.types.PrimitiveType.STRING;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.fscript.model.Model;
import org.objectweb.fractal.fscript.model.Property;
import org.objectweb.fractal.fscript.model.fractal.FractalModel;
import org.ow2.frascati.factory.runtime.domain.api.Domain;

/**
 * This class represents the SCA Fractal component model in terms of the
 * {@link Model} APIs. It describes which kinds of nodes are present in a SCA
 * architecture, what properties these nodes have, and what axes can connect these nodes
 * together to form a complete SCA architecture.
 * 
 * @author Christophe Demarey.
 */
public class FraSCAtiModel 
     extends FractalModel 
  implements NodeFactory {
  /** Names of bindings */
  public static final String DOMAIN_ITF = "domain";
  /**
   * The FraSCAti domain.
   */
  protected Domain domain;
 
  
  /**
   * Contributes the different kinds of nodes used to represent Fractal architectures.
   */
  @Override
  protected void createNodeKinds() {
    super.createNodeKinds();
    addKind( "scacomponent", 
             new Property("name", STRING, true),
             new Property("state", STRING, true) );
    addKind( "scaservice", 
             new Property("name", STRING, false),
             new Property("signature", STRING, false),
             new Property("collection", BOOLEAN, false) );
    addKind( "scareference", 
            new Property("name", STRING, false),
            new Property("signature", STRING, false),
            new Property("collection", BOOLEAN, false) );
    addKind( "scaproperty",
             new Property("name", STRING, true),
             new Property("value", OBJECT, true) );
    addKind( "scabinding",
             new Property("name", STRING, true),
             new Property("state", STRING, true) );
    addKind( "scaintent",
            new Property("name", STRING, true),
            new Property("state", STRING, true) );
  }

  /**
   * Contributes the axes which can be used to connect Fractal nodes in order to
   * represent the structure of a Fractal architecture.
   */
  @Override
  protected void createAxes() {
    super.createAxes();
    // Base axes
    addAxis(new ScaChildAxis(this));
//    addAxis(new ParentAxis(this));
//    addAxis(new ComponentAxis(this));
    addAxis(new ScaServiceAxis(this));
    addAxis(new ScaReferenceAxis(this));
    addAxis(new ScaPropertyAxis(this));
    addAxis(new ScaWireAxis(this));
    addAxis(new ScaBindingAxis(this));
    addAxis(new ScaIntentAxis(this));
    // Transitive axes
//    addAxis(new TransitiveAxis(getAxis("child"), "descendant"));
//    addAxis(new TransitiveAxis(getAxis("parent"), "ancestor"));
    // Composed axes
//    addAxis(new ComposedAxis("sibling", getAxis("parent"), getAxis("child")));
    // Reflective axes
//    addAxis(new ReflectiveAxis(getAxis("child")));
//    addAxis(new ReflectiveAxis(getAxis("parent")));
//    addAxis(new ReflectiveAxis(getAxis("descendant")));
//    addAxis(new ReflectiveAxis(getAxis("ancestor")));
//    addAxis(new ReflectiveAxis(getAxis("sibling")));
  }

  /**
   * Contributes a few custom procedures to manipulate Fractal architecture which can
   * not be described and generated in the framework of the {@link Model} APIs.
   */
  @Override
  protected void createAdditionalProcedures() {
    ScaNewAction scaNew = new ScaNewAction();

    try {
      scaNew.bindFc("frascati-model", this);
    } catch (NoSuchInterfaceException e) {
      throw new AssertionError("Internal inconsistency with ScaNewAction.");
    }
    addProcedure(scaNew);
  }
  
  /**
   * Create a new ScaComponentNode instance.
   * 
   * @param comp The SCA component to encapsulate.
   * @return the ScaComponentNode instance
   */
  public ScaComponentNode createScaComponentNode(Component comp) {
    return new ScaComponentNode(this, comp);
  }

  /**
   * Create a new ScaServiceNode instance.
   * 
   * @param comp The SCA service to encapsulate.
   * @return the ScaServiceNode instance
   */
  public ScaServiceNode createScaServiceNode(Interface itf) {
    return new ScaServiceNode(this, itf);
  }
  
  /**
   * Create a new ScaReferenceNode instance.
   * 
   * @param comp The SCA reference to encapsulate.
   * @return the ScaReferenceNode instance
   */
  public ScaReferenceNode createScaReferenceNode(Interface itf) {
    return new ScaReferenceNode(this, itf);
  }
  
  /**
   * Print this model.
   */
  @Override
  public String toString() {
    return "FraSCAti model";
  }
  
  public String[] listFc() {
    String[] tmp = super.listFc();
    String[] result = new String[tmp.length + 1];
    int idx = tmp.length;
    System.arraycopy(tmp, 0, result, 0, idx);
    
    tmp = new String[] { DOMAIN_ITF };
    System.arraycopy(tmp, 0, result, idx, tmp.length);
    return result;
  }

  public Object lookupFc(String clItfName) throws NoSuchInterfaceException {
    if (DOMAIN_ITF.equals(clItfName)) {
        return this.domain;
    } else {
      return super.lookupFc(clItfName);
    }
  }

  public void bindFc(String clItfName, Object srvItf) throws NoSuchInterfaceException {
    if  (DOMAIN_ITF.equals(clItfName)) {
        this.domain = (Domain) srvItf;
    } else {
      super.bindFc(clItfName, srvItf);
    }
  }

  public void unbindFc(String clItfName) throws NoSuchInterfaceException {
    if (DOMAIN_ITF.equals(clItfName)) {
        this.domain = null;
    } else {
      super.unbindFc(clItfName);
    }
  }

}
