/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Nicolas Dolet
 * 
 * Contributor(s): Philippe Merle
 */

package org.ow2.frascati.factory;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.util.logging.Logger;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.ow2.frascati.tinfi.TinfiDomain;

/**
 * Launcher for SCA composite applications.
 * 
 * @author Nicolas Dolet
 */
public class Launcher {
  
  /**
   * Name of the SCA composite to launch
   */
  private String compositeName;
  
  /**
   * The composite
   */
  private Component scaComposite;
  
  /**
   * Logger for this class
   */
  private Logger log = Logger.getLogger(this.getClass().getCanonicalName());
  
  //---------------------------------------------------------------------------
  // Constructor(s)
  // --------------------------------------------------------------------------
  
  /**
   * Default constructor.
   */
  public Launcher() {
    scaComposite = null;
  }

  /**
   * Launcher for a given composite.
   * 
   * @param compositeName the composite name
   */
  public Launcher(String compositeName) {
    try {
      scaComposite = new Factory().getComposite(compositeName);
      this.compositeName = compositeName;
    } catch (FactoryException e) {
      log.severe("Unable to launch " + compositeName + ": "
          + e.getMessage());
      scaComposite = null;
    }
  }
  
  /**
   * Launcher for <code>compositeName</code> which libraries
   * are in <code>urls</code>, with the Factory <code>f</code>
   * 
   * @param compositeName the name of the composite to load
   * @param f the factory to use to create composite
   * @param urls URLs of libraries required to launch the composite
   */
  public Launcher(String compositeName, Factory f, URL[] urls) {
    try {
      scaComposite = f.getComposite(compositeName,urls);
      this.compositeName = compositeName;
    } catch (RuntimeException e) {
      log.severe("Unable to launch " + compositeName + ": "
          + e.getMessage());
      scaComposite = null;
    }
  }
  
  //---------------------------------------------------------------------------
  // Public methods
  // --------------------------------------------------------------------------
  /**
   * Launch the composite
   */
  public void launch() {
    if (compositeName != null) {
      try {
        scaComposite = new Factory().getComposite(compositeName);
      } catch (FactoryException e) {
        log.severe("Unable to launch " + compositeName + ": "
            + e.getMessage());
        scaComposite = null;
      }
    } else {
      log.severe("FraSCAti launcher: composite name must be set!");
    }
  }
  
  /**
   * Launches an SCA composite
   * 
   * @param compositeName the composite to launch
   */
  public void launch(String compositeName) {
    this.compositeName = compositeName;
    launch();
  }

  /**
   * Invokes a {@link Method} on an SCA service exposed on the launched
   * composite
   * 
   * @param serviceName
   *          the service name of the SCA composite
   * @param methodName
   *          the method to invoke
   * @param responseType
   *          the response {@link Class} type
   * @param params
   *          parameters for <code>methodName</code>
   * @return the method result or null
   */
  public <T> T call(String serviceName, String methodName,
      Class<T> responseType, Object... params) {
    try {
      Object o = TinfiDomain
          .getService(scaComposite, Object.class, serviceName);
      Class<?> c = o.getClass();
      Class<?>[] paramTypes;
      String paramList = "";
      boolean withParams;
      if (params == null) {
        paramTypes = new Class[0];
        withParams = false;
      } else {
        paramTypes = new Class[params.length];
        for (int i = 0; i < params.length; ++i) {
          paramTypes[i] = params[i].getClass();
          paramList += params[i] + " ";
        }
        withParams = true;
      }
      Method m = c.getMethod(methodName, paramTypes);

      log.info("Call service '" + serviceName + "' of " + compositeName
          + " - Method '" + methodName
          + (withParams ? ("' - Params: " + paramList) : "'"));
      Object ret = m.invoke(o, params);
      @SuppressWarnings("unchecked")
      T r = (T) ret;
      return r;
    } catch (NoSuchInterfaceException e) {
      log.severe("Unable to get the service " + serviceName);
      throw new Error("Unable to get the service " + serviceName, e);
    } catch (SecurityException e) {
      log.severe("Unable to get the method " + methodName);
      throw new Error("Unable to get the method " + methodName, e);
    } catch (NoSuchMethodException e) {
      log.severe("The service " + serviceName
          + " does not provide the method " + methodName + params);
      throw new Error("The service " + serviceName
              + " does not provide the method " + methodName + params, e);
    } catch (IllegalArgumentException e) {
      log.warning(e.getMessage());
      throw new Error(e);
    } catch (IllegalAccessException e) {
      log.warning(e.getMessage());
      throw new Error(e);
    } catch (InvocationTargetException e) {
      log.warning(e.getMessage());
      throw new Error(e);
    }
  }
  
}
