/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 * Contributor: Philippe Merle
 */

package org.ow2.frascati.tinfi.control.content.scope;

import java.lang.annotation.Annotation;
import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Stack;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.julia.ComponentInterface;
import org.objectweb.fractal.julia.Interceptor;
import org.osoa.sca.CallableReference;
import org.osoa.sca.RequestContext;
import org.osoa.sca.ServiceReference;
import org.osoa.sca.annotations.Constructor;
import org.osoa.sca.annotations.Property;
import org.osoa.sca.annotations.Reference;
import org.ow2.frascati.tinfi.TinfiComponentOutInterface;
import org.ow2.frascati.tinfi.TinfiRuntimeException;
import org.ow2.frascati.tinfi.control.component.ReconfigurableComponentContext;
import org.ow2.frascati.tinfi.control.content.ContentClassMetaData;
import org.ow2.frascati.tinfi.control.content.ContentInstantiationException;
import org.ow2.frascati.tinfi.control.content.InjectionPoint;
import org.ow2.frascati.tinfi.control.content.InjectionPointFieldImpl;
import org.ow2.frascati.tinfi.control.property.SCAPropertyController;
import org.ow2.frascati.tinfi.osoa.CallbackManager;

/**
 * Root class for scope managers.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @author Philippe Merle <Philippe.Merle@inria.fr>
 */
public abstract class AbstractScopeManager implements ScopeManager {
    
    protected Component comp;
    protected SCAPropertyController scapc;
    protected ReconfigurableComponentContext compCtx;
    protected NameController nc;
    
    /**
     * The metadata for the content class associated to the current
     * scoped-component.
     */
    protected ContentClassMetaData ccmd;
    
    public AbstractScopeManager(
        Component comp, SCAPropertyController scapc,
        ReconfigurableComponentContext compCtx, NameController nc,
        ContentClassMetaData ccmd ) {
        
        this.comp = comp;
        this.scapc = scapc;
        this.compCtx = compCtx;
        this.nc = nc;
        this.ccmd = ccmd;
    }
    
    /**
     * <code>true</code> if the content class associated with {@link #ccmd} has
     * been instantiated at least once.
     * 
     * This field is used to determined if default expression associated with
     * field-based @{@link Property}s need to be registered with the property
     * controller. This registration should occur only once in order to prevent
     * unwanted loss of values which may been set latter on by invoking the
     * property controller.
     */
    private boolean instantiatedOnce = false;

    
    // ----------------------------------------------------------------------
    // Methods used by subclasses
    // ----------------------------------------------------------------------
    
    /**
     * Create and initialize a content instance.
     */
    protected Object createAndInitializeFcInstance()
    throws ContentInstantiationException {
        
        List<String> initnames = new ArrayList<String>();
        
        try {
            Object content = createFcInstance(initnames);
            initializeFcInstance(content,initnames);
            return content;
        }
        catch( Exception e ) {
            if( e instanceof ContentInstantiationException ) {
                throw (ContentInstantiationException) e;
            }
            if( e instanceof RuntimeException ) {
                throw (RuntimeException) e;
            }
            throw new ContentInstantiationException(e);
        }
    }
    
    /**
     * Create a new content instance.
     * 
     * @param initnames
     *      this list is empty when the method is called and populated with the
     *      reference and property names initialized while creating the content
     *      instance with an @{@link Constructor} annotated constructor
     */
    private Object createFcInstance( List<String> initnames )
    throws
        java.lang.InstantiationException, IllegalAccessException,
        InstantiationException, InvocationTargetException {
        
        Object content = null;
        
        if( ccmd.constructorAnnotatedElement == null ) {
            content = ccmd.fcContentClass.newInstance();
        }
        else {
            ComponentType ct = (ComponentType) comp.getFcType();

            org.osoa.sca.annotations.Constructor annot =
                ccmd.constructorAnnotatedElement.getAnnotation(
                    org.osoa.sca.annotations.Constructor.class );
            
            /*
             * @Constructor defines a String[] value property which defaults to
             * [""]. Detect this default value.
             * 
             * Potential issue: if the value is really [""]?
             */
            String[] annotvalue = annot.value();
            if( annotvalue.length>1 || annotvalue[0].length()>0 ) {
                for (String av : annotvalue) {
                    initnames.add(av);
                }
            }
                
            /*
             * Extra property names may be specified by annotating the
             * parameters of the @Constructor annotated contructor.
             */
            java.lang.reflect.Constructor<?> ctor =
                ccmd.constructorAnnotatedElement;
            Annotation[][] paramsAnnots = ctor.getParameterAnnotations();
            for (Annotation[] paramAnnots : paramsAnnots) {
                for (Annotation pannot : paramAnnots) {
                    if( pannot instanceof Property ) {
                        Property propannot = (Property) pannot;
                        String name = propannot.name();
                        initnames.add(name);
                    }
                }
            }
            
            Object[] values = new Object[initnames.size()];
            for (int i = 0 ; i < initnames.size() ; i++) {
                String name = initnames.get(i);
                try {
                    InterfaceType it = ct.getFcInterfaceType(name);
                    Object sr = getServiceReference(it);
                    values[i] = sr;
                }
                catch( NoSuchInterfaceException nsie ) {
                    /*
                     * Not a reference. Check whether this is a property.
                     */
                    if( ! scapc.containsPropertyName(name) ) {
                        /*
                         * Not a reference, nor a property.
                         */
                        String msg =
                            "Identifier "+name+
                            " in @Constructor attributes for class "+
                            ccmd.fcContentClass+
                            " is neither a reference name, nor a property name";
                        throw new InstantiationException(msg);
                    }
                    values[i] = scapc.getValue(name);
                }
            }
            
            /*
             * Construct the content instance by invoking the constructor with
             * initialization values.
             */
            ctor = ccmd.constructorAnnotatedElement;
            content = ctor.newInstance(values);
        }
        
        return content;
    }
    
    /**
     * Initialize newly created content instances.
     * 
     * @param content  the content instance
     * @param initnames
     *      the reference and property names which has been already initialized
     *      on the specified content instance
     */
    private void initializeFcInstance( Object content, List<String> initnames )
    throws
        ContentInstantiationException, IllegalAccessException,
        InvocationTargetException, NoSuchInterfaceException {
        
        /*
         * Set the component context.
         */
        if( ccmd.contextAnnotatedElement != null ) {
            ccmd.contextAnnotatedElement.set(content,compCtx);
        }
        
        /*
         * Set the component name.
         */
        if( ccmd.componentNameAnnotatedElement != null ) {
            String name = nc.getFcName();
            ccmd.componentNameAnnotatedElement.set(content,name);
        }
        
        /*
         * Set references.
         * 
         * I was previously interating on ccmd.refs.getReferenceNames() for
         * injecting references. However POJO setters are not included in this
         * set (it would be a bit difficult to decide whether they should be
         * included in ccmd.refs or in ccmd.props.)
         */
        ComponentType ct = (ComponentType) comp.getFcType();
        InterfaceType[] its = ct.getFcInterfaceTypes();
        for (InterfaceType it : its) {
            
            String name = it.getFcItfName();
            if( name.equals("component") || name.endsWith("-controller") ) {
                // Skip control interfaces
                continue;
            }
            if( ! it.isFcClientItf() ) {
                // Skip server interfaces
                continue;
            }
            
            Object sr = getServiceReference(it);
            if( ccmd.refs.containsKey(name) ) {
                // @Reference annotation
            	InjectionPoint<Reference> ip = ccmd.refs.get(name);
            	ip.set(content,sr);  // Inject the reference
                if( sr instanceof List<?> ) {
                    // Collection references
                    List<?> l = (List<?>) sr;
                    for (Object o : l) {
                        checkMandatoryReferenceHasBeenSet(o,ip,name);
                    }
                }
                else {
                    // Singleton references
                    checkMandatoryReferenceHasBeenSet(sr,ip,name);
                }
            }
            else {
                if( ccmd.setters.containsKey(name) ) {
                    // POJO
                    Method setter = ccmd.setters.get(name);
                    setter.invoke(content,sr);
                }
                else {
                    /*
                     * Indeed nothing.
                     * 
                     * Neither an annotated case, nor a POJO. The reference may
                     * have been injected via an @Constructor annotation. See
                     * {@link #createFcInstance()}.
                     */
                    if( ! initnames.contains(name) ) {
                        String msg =
                            "Reference or property "+name+
                            " can not be injected on "+ccmd.fcContentClass;
                        throw new ContentInstantiationException(msg);
                    }
                }                
            }
        }

        /*
         * Set properties.
         * 
         * I was previously iterating on ccmd.props.getReferenceNames() for
         * injecting property values. However POJO setters are not included in
         * this set (it would be a bit difficult to decide whether they should
         * be included in ccmd.refs or in ccmd.props.)
         */
        String[] propnames = scapc.getPropertyNames();
        for (String name : propnames) {
            Object value = scapc.getValue(name);
            if( ccmd.props.containsKey(name) ) {
                // @Property annotation
            	InjectionPoint<Property> ip = ccmd.props.get(name);
                ip.set(content,value);
            }
            else {
                if( ccmd.setters.containsKey(name) ) {
                    // POJO
                    Method setter = ccmd.setters.get(name);
                    setter.invoke(content,value);                
                }
                else {
                    /*
                     * Indeed nothing.
                     * 
                     * Neither an annotated case, nor a POJO. The property value
                     * may have been injected via an @Constructor annotation.
                     * See {@link #createFcInstance()}.
                     */
                    if( ! initnames.contains(name) ) {
                        String msg =
                            "Reference or property "+name+
                            " can not be injected on "+ccmd.fcContentClass;
                        throw new ContentInstantiationException(msg);
                    }
                }
            }
        }
        
        /*
         * Register all field properties with the property controller.
         * 
         * The idea is that field-based properties may have a default expression
         * in the Java code, and that this value should be registered with the
         * property controller in order for it to be able to return this value
         * when SCAPropertyController#getValue() is invoked.
         */
        Set<String> names = ccmd.props.keySet();
        if( ! instantiatedOnce ) {
            for (String name : names) {
            	if( ccmd.props.containsKey(name) ) {
            		InjectionPoint<Property> ip = ccmd.props.get(name);
            		if( ip instanceof InjectionPointFieldImpl<?> ) {
            			Object value = ip.get(content);
                        Class<?> type = ip.getType();
                        scapc.setValue(name,value);
                        scapc.setType(name,type);
            		}
            	}
            }
            instantiatedOnce = true;
        }
        
        // Check that all mandatory properties have been set
        for (String name : names) {
            if( ! scapc.containsPropertyName(name) ) {
        		InjectionPoint<Property> ip = ccmd.props.get(name);
                Property prop = ip.getAnnotation();
                if( prop.required() ) {
                    String msg =
                        "Mandatory property(s) "+name+" in component "+
                        nc.getFcName()+" should have been set";
                    throw new ContentInstantiationException(msg);
                }
            }
        }
        
        /*
         * Set callbacks.
         */
        if( ccmd.callbacks.length > 0 ) {
            CallbackManager cm = CallbackManager.get();
            if( ! cm.isEmpty() ) {
                CallableReference<?> cr = cm.peek();
                Class<?> businessItf = cr.getBusinessInterface();
                
                for (AccessibleObject ao : ccmd.callbacks) {
                    if( ao instanceof Method ) {
                        Method method = (Method) ao;
                        Class<?>[] params = method.getParameterTypes();
                        if( params[0].isAssignableFrom(businessItf) ) {
                            // initFcController checked that this was a setter
                            method.invoke(content,new Object[]{cr.getService()});
                            break;
                        }
                    }
                    else {
                        Field field = (Field) ao;
                        Class<?> type = field.getType();
                        if( type.isAssignableFrom(businessItf) ) {
                            field.set(content,cr.getService());
                            break;
                        }
                    }
                }            
            }
        }
        
        /*
         * Invoke the @Init annotated method, if any.
         */
        if( ccmd.initMethod != null ) {
            Method init = ccmd.initMethod;
            init.invoke(content);
        }
    }
    
    /**
     * If the specified reference is mandatory, check that it has been set.
     * 
     * @param sr    the service reference
     * @param name  the name of the service reference
     * @throws ContentInstantiationException if the reference has not been set
     */
    private void checkMandatoryReferenceHasBeenSet(
		Object sr, InjectionPoint<Reference> ip, String name )
    throws ContentInstantiationException {
        
		Reference ref = ip.getAnnotation();
        if ( ref.required() && sr==null ) {
            String msg =
                "Mandatory reference " + name + " in component " +
                nc.getFcName() + " should have been set";
            throw new ContentInstantiationException(msg);
        } 
    }
    
    /**
     * Return the {@link ServiceReference} (or the list of {@link
     * ServiceReference}s) associated to the specified {@link InterfaceType}.
     */
    private Object getServiceReference( InterfaceType it )
    throws NoSuchInterfaceException {
        
        String name = it.getFcItfName();
        if( it.isFcCollectionItf() ) {
            Object[] itfs = comp.getFcInterfaces();
            List<Object> srs = new ArrayList<Object>();
            for (Object o : itfs) {
                Interface itf = (Interface) o;
                String itfname = itf.getFcItfName();
                if( itfname.startsWith(name) ) {
                    Object sr = getServiceReference(itf);
                    srs.add(sr);
                }
            }
            return srs;
        }
        else {
            Object itf = comp.getFcInterface(name);
            Object sr = getServiceReference(itf);
            return sr;
        }
    }
    
    /**
     * Return a {@link ServiceReference} corresponding to the specified client
     * interface or <code>null</code> if the client interface is not bound.
     */
    private Object getServiceReference( Object clientItf ) {
        ComponentInterface itf = (ComponentInterface) clientItf;
        Interceptor outInterceptor = (Interceptor) itf.getFcItfImpl();
        Object delegate = outInterceptor.getFcItfDelegate();
        if( delegate == null ) {
        	/*
        	 * Unbound client interface. At this point this means that this is
        	 * an optional interface. Mandatory unbound interfaces are detected
        	 * when starting the component.
        	 */
        	return null;
        }
    	TinfiComponentOutInterface<?> tcoi =
    		(TinfiComponentOutInterface<?>) clientItf;
        ServiceReference<?> sr = tcoi.getServiceReference();
        return sr;
    }
    
    /**
     * Destroy the specified content instance.
     */
    protected void destroyFcInstance( Object content ) {
                
        /*
         * Invoke the @Destroy method, if any.
         */
        if( ccmd.destroyMethod != null ) {
            Method destroy = ccmd.destroyMethod;
            try {
                destroy.invoke(content);
            }
            catch( Exception e ) {
                throw new TinfiRuntimeException(e);
            }
        }
    }

    
    // ----------------------------------------------------------------------
    // Request context stack management
    // ----------------------------------------------------------------------
    
    /**
     * ThreadLocal stack holding request contexts.
     * 
     * An element is pushed on the stack by the interface (see classes generated
     * by {@link InterfaceSourceCodeGeneratorFactory}) each time an invocation
     * is received. The element is poped when the invocation has been processed
     * and just before returning the result.
     * 
     * @since 1.1.1
     */
    protected ThreadLocal<Stack<RequestContext>> tl =
        new ThreadLocal<Stack<RequestContext>>();

    /**
     * Push a new request context on the request context stack. This request
     * context is returned by {@link
     * org.osoa.sca.ComponentContext#getRequestContext()}. The stack is needed
     * in case of reentrant component operation invocations.
     * 
     * @param rc  the request context to be pushed
     * @since 1.1.1
     */
    public void pushRequestContext( RequestContext rc ) {
        Stack<RequestContext> stack = tl.get();
        if( stack == null ) {
            /*
             * The {@link #tl} field can not be initialized with a default
             * expression since its value is thread-specific.
             */
            stack = new Stack<RequestContext>();
            tl.set(stack);
        }
        stack.push(rc);
    }
    
    /**
     * Pop the previously pushed request context.
     * 
     * @return  the poped request context
     * @since 1.1.1
     */
    public RequestContext popRequestContext() {
        Stack<RequestContext> stack = tl.get();
        RequestContext rc = stack.pop();
        return rc;
    }
    
    /**
     * Return the request context on top of the request context stack.
     * 
     * @return  the request context on top of the request context stack
     * @since 1.1.1
     */
    public RequestContext peekRequestContext() {
        Stack<RequestContext> stack = tl.get();
        RequestContext rc = stack.peek();
        return rc;
    }
}
