/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.reflect;

import java.lang.annotation.Annotation;
import java.lang.reflect.AccessibleObject;
import java.lang.reflect.AnnotatedElement;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.juliac.runtime.ClassLoaderFcItf;
import org.objectweb.fractal.juliac.runtime.ClassLoaderItf;
import org.objectweb.fractal.juliac.runtime.Juliac;
import org.ow2.frascati.tinfi.TinfiRuntimeException;
import org.ow2.frascati.tinfi.reflect.AnnotatedElementFilter;
import org.ow2.frascati.tinfi.reflect.Filter;
import org.ow2.frascati.tinfi.reflect.Filters;
import org.ow2.frascati.tinfi.reflect.SetterMethodFilter;
import org.ow2.frascati.tinfi.reflect.UnAnnotatedElementFilter;

/**
 * Utility methods.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class Util {

    /**
     * Check whether the specified method is a getter.
     * 
     * @param getter  the method to be checked
     * @throws IllegalArgumentException
     *      if <code>getter</code> is not a getter method
     * @since 1.1.2
     */
    public static void checkGetterMethod( Method getter )
    throws IllegalArgumentException {        

        String name = getter.getName();
        if( ! name.startsWith("get") ) {
            String msg =
                "The name of a getter method should start with get: "+getter;
            throw new IllegalArgumentException(msg);
        }
        Class<?>[] ptypes = getter.getParameterTypes();
        if( ptypes.length != 0 ) {
            String msg =
                "A getter method should not define any parameter: "+getter;
            throw new IllegalArgumentException(msg);
        }
    }

    /**
     * Check whether the specified methods are a valid pair of setter/getter
     * methods for the same property.
     * 
     * @throws IllegalArgumentException  if this is not the case
     * @since 1.1.2
     */
    public static void checkMatchingSetterGetterMethods(
		Method setter, Method getter )
    throws IllegalArgumentException {
    	
    	SetterMethodFilter.checkSetterMethod(setter);
    	checkGetterMethod(getter);
    	
    	String setterPropName = SetterMethodFilter.getSetterPropertyName(setter);
    	String getterPropName = getGetterPropertyName(getter);
    	if( ! setterPropName.equals(getterPropName) ) {
    		String msg =
    			"Property names differ: "+setterPropName+" vs "+getterPropName;
    		throw new IllegalArgumentException(msg);
    	}
    	
    	Class<?> setterPropType = SetterMethodFilter.getSetterPropertyType(setter);
    	Class<?> getterPropType = getGetterPropertyType(getter);
    	if( ! setterPropType.equals(getterPropType) ) {
    		String msg =
    			"Property types differ: "+setterPropType+" vs "+getterPropType;
    		throw new IllegalArgumentException(msg);
    	}
    }
    
    /**
     * Return all fields, declared ones whatever their access modifier is and
     * inherited public ones, in the specified class.
     * 
     * @param cl  the class
     * @since 1.1.2
     */
    public static Field[] getAllFields( Class<?> cl ) {
    	
    	/*
    	 * Use a set to accomodate duplicate insertions of public declared
    	 * elements, e.g. public methods which are returned both by
    	 * getDeclaredMethods and getMethods.
    	 */
    	Set<Field> result = new HashSet<Field>();
    	
    	// Declared fields
    	Field[] fields = cl.getDeclaredFields();
    	for (Field field : fields) {
			result.add(field);
		}
    	
    	// Inherited public fields
    	fields = cl.getFields();
    	for (Field field : fields) {
			result.add(field);
		}
    	
    	return result.toArray(new Field[result.size()]);
    }
    
    /**
     * Return all methods, declared ones whatever their access modifier is and
     * inherited public ones, in the specified class.
     * 
     * @param cl  the class
     * @since 1.1.2
     */
    public static Method[] getAllMethods( Class<?> cl ) {
    	
    	/*
    	 * Use a set instead to accomodate duplicate insertions of public
    	 * declared elements, e.g. public methods which are returned both by
    	 * getDeclaredMethods and getMethods.
    	 */
    	Set<Method> result = new HashSet<Method>();
    	
    	// Declared methods
    	Method[] methods = cl.getDeclaredMethods();
    	for (Method method : methods) {
			result.add(method);
		}
    	
    	// Inherited public methods
    	methods = cl.getMethods();
    	for (Method method : methods) {
			result.add(method);
		}
    	
    	return result.toArray(new Method[result.size()]);
    }
    
    /**
     * Return all setter methods and fields annotated with the specified
     * annotation type in the specified class. The matching elements declared in
     * the specified type, whatever their access modifier is, and the public
     * inherited ones are returned.
     * 
     * @param cl    the class
     * @param type  the annotation type
     * @since 1.2
     */
    public static AccessibleObject[] getAllAnnotatedSettersAndFields(
		Class<?> cl, Class<? extends Annotation> type ) {
    	
    	// Setter methods
    	Method[] methods = getAllMethods(cl);
    	Method[] ms0 = Filters.filter(methods,new SetterMethodFilter());
    	Filter<AnnotatedElement> filter = new AnnotatedElementFilter(type);
    	Method[] ms1 = Filters.filter(ms0,filter);
    	
    	// Fields
    	Field[] fields = getAllFields(cl);
    	Field[] fs0 = Filters.filter(fields,filter);

    	// Result
    	AccessibleObject[] aos =
    		new AccessibleObject[ ms1.length + fs0.length ];
    	System.arraycopy(ms1,0,aos,0,ms1.length);
    	System.arraycopy(fs0,0,aos,ms1.length,fs0.length);
    	
    	return aos;
    }
    
    /**
     * Return the getter method corresponding to the specified setter.
     * 
     * @throws NoSuchMethodException  if the getter does not exist
     * @since 1.1.2
     */
    public static Method getGetterForSetter( Method setter )
    throws NoSuchMethodException {
    	
    	Class<?> cl = setter.getDeclaringClass();
    	
    	// First search in declared methods
    	Method[] methods = cl.getDeclaredMethods();
    	for (Method method : methods) {
			if( isMatchingSetterGetter(setter,method) ) {
				return method;
			}
		}

    	// First search in declared methods
    	methods = cl.getMethods();
    	for (Method method : methods) {
			if( isMatchingSetterGetter(setter,method) ) {
				return method;
			}
		}
    	
    	throw new NoSuchMethodException();
    }
    
    /**
     * Return the name of the property set by the specified getter method.
     * 
     * @param getter  a getter method
     * @return        the name of the property set by <code>getter</code>
     * @throws IllegalArgumentException
     *      if <code>getter</code> is not a getter method
     */
    public static String getGetterPropertyName( Method getter )
    throws IllegalArgumentException {
        
        checkGetterMethod(getter);
        
        String name = getter.getName();
        name = name.substring(3);   // Skip get
        name = name.substring(0,1).toLowerCase() + name.substring(1);
        
        return name;
    }
    /**
     * Return the type of the property set by the specified getter method.
     * 
     * @param getter  a setter method
     * @return        the type of the property set by <code>getter</code>
     * @throws IllegalArgumentException
     *      if <code>getter</code> is not a getter method
     * @since 1.1.2
     */
    public static Class<?> getGetterPropertyType( Method getter )
    throws IllegalArgumentException {
        
        checkGetterMethod(getter);
        
        Class<?> rtype = getter.getReturnType();
        return rtype;
    }

    /**
     * Return all unannotated methods, declared ones whatever their access
     * modifier is and inherited public ones, in the specified class.
     * 
     * @param cl  the class
     * @return    the corresponding map of methods indexed by property names
     *            (the property set by the method)
     * @since 1.1.2
     */
    public static Map<String,Method> getAllUnAnnotatedSetterMethods(
		Class<?> cl ) {
        
    	Method[] methods = getAllMethods(cl);
    	Method[] ms0 = Filters.filter(methods,new SetterMethodFilter());
    	Filter<AnnotatedElement> filter = new UnAnnotatedElementFilter();
    	Method[] ms1 = Filters.filter(ms0,filter);
        
    	// Create the resulting map
        Map<String,Method> result = new HashMap<String,Method>();
        for (Method setter : ms1) {
			String propName = SetterMethodFilter.getSetterPropertyName(setter);
			result.put(propName,setter);
		}

        return result;
    }

    /**
     * Return <code>true</code> if the specified method is a getter.
     * 
     * @param getter  the method to be checked
     * @return  <code>true</code> if this is a getter
     */
    public static boolean isGetterMethod( Method getter ) {
        try {
            checkGetterMethod(getter);
            return true;
        }
        catch( IllegalArgumentException iae ) {
            return false;
        }
    }
    
    /**
     * Return <code>true</code> if the specified pair of methods are valid
     * setter/getter for the same property.
     */
    public static boolean isMatchingSetterGetter(
		Method setter, Method getter ) {
    	
    	try {
    		checkMatchingSetterGetterMethods(setter,getter);
    		return true;
    	}
    	catch( IllegalArgumentException iae ) {
    		return false;
    	}
    }

    /**
     * Load the class whose name is specified with the specified classloader.
     * 
     * @throws ClassNotFoundException  if the class can not be loaded
     */
    public static Class<?> loadClass( String name, ClassLoader cl )
    throws ClassNotFoundException {
        Class<?> c = cl.loadClass(name);
        return c;
    }

    /**
     * Load the class whose name is specified with the classloader interface
     * provided by the Juliac bootstrap component.
     * 
     * @throws ClassNotFoundException  if the class can not be loaded
     */
    public static Class<?> loadClass( String name )
    throws ClassNotFoundException {
    
        ClassLoaderItf cl = null;
        try {
            Component boot = new Juliac().newFcInstance();
            cl = (ClassLoaderItf) boot.getFcInterface(ClassLoaderFcItf.NAME);
        }
        catch( NoSuchInterfaceException nsie ) {
            /*
             * Shouldn't occur since Juliac provides a classloader interface.
             */
            throw new TinfiRuntimeException(nsie);
        }
        
        Class<?> c = cl.loadClass(name);
        return c;
    }
}
